Package: admisc
Version: 0.40
SystemRequirements: OpenMP (optional)
Title: Adrian Dusa's Miscellaneous
Authors@R: person(
    given = "Adrian", family = "Dusa",
    role = c("aut", "cre", "cph"),
    email = "dusa.adrian@unibuc.ro",
    comment = c(ORCID = "0000-0002-3525-9253"))
URL: https://github.com/dusadrian/admisc
BugReports: https://github.com/dusadrian/admisc/issues
Depends: R (>= 3.5.0)
Imports: methods
Suggests: QCA (>= 3.7)
Description: Contains functions used across packages 'DDIwR', 'QCA' and 'venn'.
    Interprets and translates, factorizes and negates SOP - Sum of Products
    expressions, for both binary and multi-value crisp sets, and extracts
    information (set names, set values) from those expressions. Other functions
    perform various other checks if possibly numeric (even if all numbers reside
    in a character vector) and coerce to numeric, or check if the numbers are
    whole. It also offers, among many others, a highly versatile recoding
    routine and some more flexible alternatives to the base functions 'with()'
    and 'within()'.
    SOP simplification functions in this package use related minimization from
    package 'QCA', which is recommended to be installed despite not being listed
    in the Imports field, due to circular dependency issues.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-03-26 18:14:59 UTC; dusadrian
Author: Adrian Dusa [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-3525-9253>)
Maintainer: Adrian Dusa <dusa.adrian@unibuc.ro>
Repository: CRAN
Date/Publication: 2026-03-27 06:10:17 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 09:56:04 UTC; unix
Archs: admisc.so.dSYM
