# Generated by roxygen2: do not edit by hand

S3method("[",admisc_fobject)
S3method(asNumeric,declared)
S3method(asNumeric,default)
S3method(asNumeric,factor)
S3method(change,QCA_tt)
S3method(change,default)
S3method(export,data.frame)
S3method(export,default)
S3method(export,list)
S3method(inside,data.frame)
S3method(inside,list)
S3method(print,admisc_deMorgan)
S3method(print,admisc_factorize)
S3method(print,admisc_fobject)
S3method(print,admisc_intersection)
S3method(print,admisc_simplify)
S3method(print,admisc_translate)
S3method(recode,declared)
S3method(recode,default)
S3method(update,character)
S3method(using,data.frame)
S3method(using,default)
S3method(using,matrix)
export(aeqb)
export(agtb)
export(agteb)
export(altb)
export(alteb)
export(aneqb)
export(anyTagged)
export(asNumeric)
export(asSOP)
export(betweenBrackets)
export(betweenQuotes)
export(change)
export(checkMV)
export(checkSubset)
export(checkValid)
export(classify)
export(coerceMode)
export(combnk)
export(compute)
export(curlyBrackets)
export(dashes)
export(deMorgan)
export(doublequotes)
export(expand)
export(expandBrackets)
export(export)
export(factorize)
export(frelevel)
export(frev)
export(getInfo)
export(getLevels)
export(getMatrix)
export(getName)
export(getNonChars)
export(getTag)
export(hasTag)
export(hastilde)
export(hclr)
export(inside)
export(insideBrackets)
export(intersection)
export(invert)
export(listRDA)
export(makeTag)
export(mvSOP)
export(negate)
export(notilde)
export(numdec)
export(objRDA)
export(outsideBrackets)
export(overwrite)
export(padBoth)
export(padLeft)
export(padRight)
export(permutations)
export(possibleNumeric)
export(prettyString)
export(prettyTable)
export(recode)
export(recreate)
export(reload)
export(removeSingleStars)
export(replaceText)
export(roundBrackets)
export(scan.clipboard)
export(setColnames)
export(setDimnames)
export(setRownames)
export(simplify)
export(simplifyList)
export(singlequotes)
export(solveBrackets)
export(sop)
export(sortExpressions)
export(spaces)
export(splitBrackets)
export(splitMainComponents)
export(splitPluses)
export(splitStars)
export(splitTildas)
export(splitstr)
export(squareBrackets)
export(stopError)
export(tildae)
export(tilde1st)
export(translate)
export(trimstr)
export(tryCatchWEM)
export(uninstall)
export(unload)
export(using)
export(validateNames)
export(verify)
export(wholeNumeric)
export(write.clipboard)
export(writePIs)
export(writePrimeimp)
importFrom(grDevices,hcl)
importFrom(methods,is)
importFrom(stats,dist)
importFrom(stats,na.omit)
importFrom(stats,relevel)
importFrom(utils,capture.output)
importFrom(utils,compareVersion)
importFrom(utils,installed.packages)
importFrom(utils,packageDescription)
importFrom(utils,read.csv)
importFrom(utils,remove.packages)
importFrom(utils,tail)
importFrom(utils,write.csv)
importFrom(utils,write.table)
useDynLib(admisc, .registration = TRUE)
