# Copyright (c) 2019 - 2026, Adrian Dusa
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, in whole or in part, are permitted provided that the
# following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * The names of its contributors may NOT be used to endorse or promote
#       products derived from this software without specific prior written
#       permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL ADRIAN DUSA BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#' Extract information between quotes in a string
#'
#' Functions to extract the between the (escaped) quotes, in a string.
#'
#' @name betweenQuotes
#' @rdname betweenQuotes
#' @rawRd
#' \usage{
#' betweenQuotes(x)
#' }
#'
#' \arguments{
#'   \item{x}{A string.}
#' }
#'
#'
#' \author{
#' Adrian Dusa
#' }
#'
#' \examples{
#' x <- "An example of \"quoted\" text."
#'
#' betweenQuotes(x)
#' }
#'
#' \keyword{functions}
NULL
#' @export
`betweenQuotes` <- function(x) {
    pos <- gregexpr("\"", x)
    lpos <- length(pos[[1]])
    if (lpos == 0) {
        return("")
    }
    else if (lpos%%2 != 0) {
        stopError("Odd number of quotes")
    }
    else {
        pos <- pos[[1]]
        result <- character(lpos)
        for (i in seq(1, lpos, by = 2)) {
            result[i] <- substr(x, pos[i] + 1, pos[i + 1] - 1)
        }
        return(result[nchar(result) > 0])
    }
}
