# Copyright (c) 2019 - 2026, Adrian Dusa
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, in whole or in part, are permitted provided that the
# following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * The names of its contributors may NOT be used to endorse or promote
#       products derived from this software without specific prior written
#       permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL ADRIAN DUSA BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#' admisc internal functions
#'
#' Functions to be used internally in package `admisc`.
#'
#' @name admisc_internal
#' @aliases anyTagged
#' @aliases checkMV
#' @aliases checkSubset
#' @aliases classify
#' @aliases dashes
#' @aliases doublequotes
#' @aliases expandBrackets
#' @aliases getInfo
#' @aliases getLevels
#' @aliases getMatrix
#' @aliases getNonChars
#' @aliases getTag
#' @aliases hasTag
#' @aliases makeTag
#' @aliases negateLoop
#' @aliases padLeft
#' @aliases padRight
#' @aliases padBoth
#' @aliases prettyString
#' @aliases prettyTable
#' @aliases reload
#' @aliases removeSingleStars
#' @aliases splitMainComponents
#' @aliases splitstr
#' @aliases splitBrackets
#' @aliases splitPluses
#' @aliases splitProducts
#' @aliases splitStars
#' @aliases splitTildas
#' @aliases solveBrackets
#' @aliases sortExpressions
#' @aliases simplifyList
#' @aliases singlequotes
#' @aliases spaces
#' @aliases stopError
#' @aliases tildae
#' @aliases trimstr
#' @aliases uninstall
#' @aliases unload
#' @aliases checkValid
#' @aliases validateNames
#' @aliases verify
#' @aliases writePIs
#' @aliases writePrimeimp
#' @keywords internal
NULL
