# Copyright (c) 2019 - 2026, Adrian Dusa
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, in whole or in part, are permitted provided that the
# following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * The names of its contributors may NOT be used to endorse or promote
#       products derived from this software without specific prior written
#       permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL ADRIAN DUSA BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#' Cross platform scan/write clipboard
#'
#' Functions to read and write to the system's clipboard, for copy/paste operations.
#'
#' @name scan.clipboard
#' @rdname clipboard
#' @aliases write.clipboard
#' @rawRd
#' \usage{
#' scan.clipboard(...)
#' write.clipboard(x)
#' }
#'
#' \arguments{
#'     \item{x}{Object to be written to the clipboard}
#'     \item{...}{Same arguments that are used in the base function \bold{\code{scan}}}
#' }
#'
#'
#' \author{
#' Adrian Dusa
#' }
#'
#' \keyword{functions}
NULL
#' @export
scan.clipboard <- function (...) {
    dots <- list(...)
    if (Sys.info()[['sysname']] == "Darwin") {
        clipboard <- readLines(textConnection(system("pbpaste", intern = TRUE)))
        sep <- ifelse(is.null(dots$sep), "\t", dots$sep)
        clipboard <- unlist(strsplit(clipboard, split = sep))
    } else if (Sys.info()[['sysname']] == "Windows") {
        dots$file <- "clipboard"
        clipboard <- do.call("scan", dots)
    }
    clipboard <- clipboard[clipboard != ""]
    if (possibleNumeric(clipboard)) {
        return(asNumeric(clipboard))
    } else {
        return(clipboard)
    }
}
