# Copyright (c) 2019 - 2026, Adrian Dusa
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, in whole or in part, are permitted provided that the
# following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * The names of its contributors may NOT be used to endorse or promote
#       products derived from this software without specific prior written
#       permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL ADRIAN DUSA BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#' Coerce an atomic vector to numeric or integer, if possible
#'
#' This function verifies if an R vector is possibly numeric, and further if the
#' numbers inside are whole numbers.
#'
#' @name coerceMode
#' @rdname coerceMode
#' @rawRd
#' \usage{
#' coerceMode(x)
#' }
#'
#' \arguments{
#'     \item{x}{An atomic R vector}
#' }
#'
#' \value{
#'     An R vector of coerced mode.
#' }
#'
#' \author{
#' Adrian Dusa
#' }
#'
#' \examples{
#' obj <- c("1.0", 2:5)
#'
#' is.integer(coerceMode(obj))
#' }
#'
#' \keyword{functions}
NULL
#' @export
`coerceMode` <- function(x) {
    if (!is.atomic(x)) {
        stopError("The input is not atomic.")
    }
    if (
        !is.numeric(x) && 
        (possibleNumeric(x) || all(is.na(x)))
    ) {
        x <- asNumeric(x)
    }
    if (
        !is.integer(x) &&
        wholeNumeric(x) &&
        is.null(tryCatchWEM(as.integer(x)))
    ) {
        x <- as.integer(x)
    }
    return(x)
}
